import { testDb } from './setup.js';
import { groups, tasks, steps, images, users, notes, printHistory } from '../index.js';
import type { Group, Task, Step, Image, User, Note, PrintHistory } from '@shared/index';

describe('Database Operations', () => {
  beforeEach(async () => {
    // Clean up the database before each test
    await images(testDb).delete();
    await steps(testDb).delete();
    await tasks(testDb).delete();
    await groups(testDb).delete();
    await users(testDb).delete();
    await notes(testDb).delete();
    await printHistory(testDb).delete();
  });

  afterAll(async () => {
    // Close the database connection after all tests
    await testDb.destroy();
  });

  describe('Groups', () => {
    it('should create and retrieve a group', async () => {
      const groupData: Omit<Group, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test Group',
      };

      const [id] = await groups(testDb).insert(groupData);
      const group = await groups(testDb).where({ id }).first();

      expect(group).toBeDefined();
      expect(group?.name).toBe(groupData.name);
    });

    it('should create a nested group', async () => {
      const parentGroupData: Omit<Group, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Parent Group',
      };

      const [_parentId] = await groups(testDb).insert(parentGroupData);

      const childGroupData: Omit<Group, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Child Group',
      };

      const [childId] = await groups(testDb).insert(childGroupData);
      const childGroup = await groups(testDb).where({ id: childId }).first();

      expect(childGroup).toBeDefined();
      expect(childGroup?.name).toBe(childGroupData.name);
    });
  });

  describe('Tasks', () => {
    it('should create and retrieve a task', async () => {
      const groupData: Omit<Group, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test Group',
      };

      const [groupId] = await groups(testDb).insert(groupData);

      const taskData: Omit<Task, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test Task',
        group_id: groupId,
        print_count: 0,
        last_printed_at: undefined,
      };

      const [id] = await tasks(testDb).insert(taskData);
      const task = await tasks(testDb).where({ id }).first();

      expect(task).toBeDefined();
      expect(task?.name).toBe(taskData.name);
      expect(task?.group_id).toBe(groupId);
    });
  });

  describe('Steps', () => {
    it('should create and retrieve a step', async () => {
      const groupData: Omit<Group, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test Group',
      };

      const [groupId] = await groups(testDb).insert(groupData);

      const taskData: Omit<Task, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test Task',
        group_id: groupId,
        print_count: 0,
        last_printed_at: undefined,
      };

      const [taskId] = await tasks(testDb).insert(taskData);

      const stepData: Omit<Step, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test Step',
        instructions: 'Test Description',
        task_id: taskId,
        order: 1,
        print_count: 0,
        last_printed_at: undefined,
      };

      const [id] = await steps(testDb).insert(stepData);
      const step = await steps(testDb).where({ id }).first();

      expect(step).toBeDefined();
      expect(step?.name).toBe(stepData.name);
      expect(step?.instructions).toBe(stepData.instructions);
      expect(step?.task_id).toBe(taskId);
    });
  });

  describe('Images', () => {
    it('should create and retrieve an image', async () => {
      const groupData: Omit<Group, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test Group',
      };

      const [groupId] = await groups(testDb).insert(groupData);

      const taskData: Omit<Task, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test Task',
        group_id: groupId,
        print_count: 0,
        last_printed_at: undefined,
      };

      const [taskId] = await tasks(testDb).insert(taskData);

      const stepData: Omit<Step, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test Step',
        instructions: 'Test Description',
        task_id: taskId,
        order: 1,
        print_count: 0,
        last_printed_at: undefined,
      };

      const [stepId] = await steps(testDb).insert(stepData);

      const imageData: Omit<Image, 'id' | 'created_at' | 'updated_at'> = {
        step_id: stepId,
        original_path: '/path/to/image.png',
        bw_path: '/path/to/image.bw.png',
        order: 1,
      };

      const [id] = await images(testDb).insert(imageData);
      const image = await images(testDb).where({ id }).first();

      expect(image).toBeDefined();
      expect(image?.step_id).toBe(stepId);
      expect(image?.original_path).toBe(imageData.original_path);
      expect(image?.bw_path).toBe(imageData.bw_path);
      expect(image?.order).toBe(imageData.order);
    });
  });

  describe('Users', () => {
    it('should create and retrieve a user', async () => {
      const userData: Omit<User, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test User',
      };

      const [id] = await users(testDb).insert(userData);
      const user = await users(testDb).where({ id }).first();

      expect(user).toBeDefined();
      expect(user?.name).toBe(userData.name);
    });
  });

  describe('Notes', () => {
    it('should create and retrieve a note for a step', async () => {
      const groupData: Omit<Group, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test Group',
      };

      const [groupId] = await groups(testDb).insert(groupData);

      const taskData: Omit<Task, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test Task',
        group_id: groupId,
        print_count: 0,
        last_printed_at: undefined,
      };

      const [taskId] = await tasks(testDb).insert(taskData);

      const stepData: Omit<Step, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test Step',
        instructions: 'Test Description',
        task_id: taskId,
        order: 1,
        print_count: 0,
        last_printed_at: undefined,
      };

      const [stepId] = await steps(testDb).insert(stepData);

      const userData: Omit<User, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test User',
      };

      const [userId] = await users(testDb).insert(userData);

      const noteData: Omit<Note, 'id' | 'created_at' | 'updated_at'> = {
        content: 'Test note for step',
        step_id: stepId,
        created_by: userId,
        task_id: undefined,
      };

      const [id] = await notes(testDb).insert(noteData);
      const note = await notes(testDb).where({ id }).first();

      expect(note).toBeDefined();
      expect(note?.content).toBe(noteData.content);
      expect(note?.step_id).toBe(stepId);
      expect(note?.created_by).toBe(userId);
      expect(note?.task_id == null).toBe(true);
    });
  });

  describe('PrintHistory', () => {
    it('should create and retrieve a print history record for a task', async () => {
      const groupData: Omit<Group, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test Group',
      };

      const [groupId] = await groups(testDb).insert(groupData);

      const taskData: Omit<Task, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test Task',
        group_id: groupId,
        print_count: 0,
        last_printed_at: undefined,
      };

      const [taskId] = await tasks(testDb).insert(taskData);

      const userData: Omit<User, 'id' | 'created_at' | 'updated_at'> = {
        name: 'Test User',
      };

      const [userId] = await users(testDb).insert(userData);

      const printHistoryData: Omit<PrintHistory, 'id' | 'created_at' | 'updated_at'> = {
        task_id: taskId,
        user_id: userId,
        printed_at: new Date(),
      };

      const [id] = await printHistory(testDb).insert(printHistoryData);
      const record = await printHistory(testDb).where({ id }).first();

      expect(record).toBeDefined();
      expect(record?.task_id).toBe(taskId);
      expect(record?.user_id).toBe(userId);
    });
  });
});