import { Knex } from 'knex';
import logger from '../../logger';

export abstract class BaseRepository<T extends { created_at?: Date; updated_at?: Date }> {
  protected tableName: string;
  protected db: Knex;
  protected items: T[] = [];

  constructor(db: Knex, tableName: string) {
    this.db = db;
    this.tableName = tableName;
  }

  async findAll(): Promise<T[]> {
    return await this.db(this.tableName).select('*');
  }

  async findById(id: number): Promise<T | null> {
    const result = await this.db(this.tableName).where('id', id).first();
    logger.info(`findById(${id}) for table ${this.tableName}:`, result);
    return result || null;
  }

  async create(data: Omit<T, 'id' | 'created_at' | 'updated_at'> & Partial<Pick<T, 'created_at' | 'updated_at'>>): Promise<T> {
    const now = new Date();
    const created_at = (data.created_at ?? now).toISOString();
    const updated_at = (data.updated_at ?? now).toISOString();
    const row = {
      ...data,
      created_at,
      updated_at,
    };
    const [id] = await this.db(this.tableName).insert(row);
    return await this.findById(id) as T;
  }

  async update(id: number, data: Partial<T>): Promise<boolean> {
    const count = await this.db(this.tableName).where('id', id).update(data);
    return count > 0;
  }

  async delete(id: number): Promise<boolean> {
    const count = await this.db(this.tableName).where('id', id).delete();
    return count > 0;
  }
}