import { BarcodeUtils, BarcodeData } from '../barcode-utils';

describe('BarcodeUtils', () => {
  describe('parseBarcode', () => {
    it('should parse valid task barcode', () => {
      const result = BarcodeUtils.parseBarcode('100000123');
      expect(result).toEqual({ type: 'TASK', id: 123 });
    });

    it('should parse valid step barcode', () => {
      const result = BarcodeUtils.parseBarcode('200000456');
      expect(result).toEqual({ type: 'STEP', id: 456 });
    });

    it('should parse barcode with non-numeric characters', () => {
      const result = BarcodeUtils.parseBarcode('1ABC000123DEF');
      expect(result).toEqual({ type: 'TASK', id: 123 });
    });

    it('should return null for invalid entity type', () => {
      const result = BarcodeUtils.parseBarcode('300000123');
      expect(result).toBeNull();
    });

    it('should return null for too short barcode', () => {
      const result = BarcodeUtils.parseBarcode('1');
      expect(result).toBeNull();
    });

    it('should return null for invalid ID', () => {
      const result = BarcodeUtils.parseBarcode('100000000');
      expect(result).toBeNull();
    });

    it('should return null for empty string', () => {
      const result = BarcodeUtils.parseBarcode('');
      expect(result).toBeNull();
    });

    it('should parse large IDs correctly', () => {
      const result = BarcodeUtils.parseBarcode('199999999');
      expect(result).toEqual({ type: 'TASK', id: 99999999 });
    });
  });

  describe('generateTaskBarcode', () => {
    it('should generate task barcode', () => {
      const result = BarcodeUtils.generateTaskBarcode(123);
      expect(result).toBe('100000123');
    });

    it('should generate task barcode with padding', () => {
      const result = BarcodeUtils.generateTaskBarcode(1);
      expect(result).toBe('100000001');
    });
  });

  describe('generateStepBarcode', () => {
    it('should generate step barcode', () => {
      const result = BarcodeUtils.generateStepBarcode(456);
      expect(result).toBe('200000456');
    });

    it('should generate step barcode with padding', () => {
      const result = BarcodeUtils.generateStepBarcode(1);
      expect(result).toBe('200000001');
    });
  });

  describe('isValidBarcode', () => {
    it('should return true for valid task barcode', () => {
      const result = BarcodeUtils.isValidBarcode('100000123');
      expect(result).toBe(true);
    });

    it('should return true for valid step barcode', () => {
      const result = BarcodeUtils.isValidBarcode('200000456');
      expect(result).toBe(true);
    });

    it('should return false for invalid barcode', () => {
      const result = BarcodeUtils.isValidBarcode('300000123');
      expect(result).toBe(false);
    });
  });

  describe('getEntityType', () => {
    it('should return TASK for task barcode', () => {
      const result = BarcodeUtils.getEntityType('100000123');
      expect(result).toBe('TASK');
    });

    it('should return STEP for step barcode', () => {
      const result = BarcodeUtils.getEntityType('200000456');
      expect(result).toBe('STEP');
    });

    it('should return null for invalid barcode', () => {
      const result = BarcodeUtils.getEntityType('300000123');
      expect(result).toBeNull();
    });
  });

  describe('getEntityId', () => {
    it('should return ID for task barcode', () => {
      const result = BarcodeUtils.getEntityId('100000123');
      expect(result).toBe(123);
    });

    it('should return ID for step barcode', () => {
      const result = BarcodeUtils.getEntityId('200000456');
      expect(result).toBe(456);
    });

    it('should return null for invalid barcode', () => {
      const result = BarcodeUtils.getEntityId('300000123');
      expect(result).toBeNull();
    });
  });

  describe('getMaxId', () => {
    it('should return maximum supported ID', () => {
      const result = BarcodeUtils.getMaxId();
      expect(result).toBe(99999999);
    });
  });

  describe('isValidId', () => {
    it('should return true for valid ID', () => {
      const result = BarcodeUtils.isValidId(123);
      expect(result).toBe(true);
    });

    it('should return false for zero ID', () => {
      const result = BarcodeUtils.isValidId(0);
      expect(result).toBe(false);
    });

    it('should return false for negative ID', () => {
      const result = BarcodeUtils.isValidId(-1);
      expect(result).toBe(false);
    });

    it('should return false for ID too large', () => {
      const result = BarcodeUtils.isValidId(100000000);
      expect(result).toBe(false);
    });
  });

  describe('round-trip tests', () => {
    it('should generate and parse task barcode correctly', () => {
      const taskId = 789;
      const generated = BarcodeUtils.generateTaskBarcode(taskId);
      const parsed = BarcodeUtils.parseBarcode(generated);

      expect(parsed).toEqual({ type: 'TASK', id: taskId });
    });

    it('should generate and parse step barcode correctly', () => {
      const stepId = 101;
      const generated = BarcodeUtils.generateStepBarcode(stepId);
      const parsed = BarcodeUtils.parseBarcode(generated);

      expect(parsed).toEqual({ type: 'STEP', id: stepId });
    });
  });
});