import { CommandBuilder } from '../printer-commands';
import { TestCommandExecutor } from '../command-executor';

describe('Command System (Standalone)', () => {
  let commandExecutor: TestCommandExecutor;

  beforeEach(() => {
    commandExecutor = new TestCommandExecutor();
  });

  it('should execute text commands', async () => {
    const commands = [
      CommandBuilder.text('Hello World'),
      CommandBuilder.newline(),
      CommandBuilder.text('Test'),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('Hello World');
    expect(output).toContain('Test');
  });

  it('should execute header commands', async () => {
    const commands = [
      CommandBuilder.header('Test Header'),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('[ ] Test Header');
  });

  it('should execute banner commands', async () => {
    const commands = [
      CommandBuilder.banner('=', 10),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('==========');
  });

  it('should execute barcode commands', async () => {
    const commands = [
      CommandBuilder.barcode('123456'),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('[BARCODE: 123456]');
  });

  it('should execute section commands', async () => {
    const commands = [
      CommandBuilder.section('Test Section', 'Test Content', '-', true),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('[ ] Test Section');
    expect(output).toContain('Test Content');
  });

  it('should execute step header commands', async () => {
    const commands = [
      CommandBuilder.stepHeader('Test Step', 1, 'Test Task', true),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('[ ] Step 1: Test Step');
  });

  it('should execute cut commands', async () => {
    const commands = [
      CommandBuilder.cut(true, 4),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('--- CUT ---');
  });

  it('should execute complex command sequences', async () => {
    const commands = [
      CommandBuilder.header('Complex Test'),
      CommandBuilder.banner('=', 20),
      CommandBuilder.text('Some content'),
      CommandBuilder.newline(),
      CommandBuilder.barcode('123'),
      CommandBuilder.cut(),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('[ ] Complex Test');
    expect(output).toContain('====================');
    expect(output).toContain('Some content');
    expect(output).toContain('[BARCODE: 123]');
    expect(output).toContain('--- CUT ---');
  });

  it('should handle empty command arrays', async () => {
    const commands: any[] = [];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toBe('\n');
  });

  it('should handle list commands', async () => {
    const commands = [
      CommandBuilder.list(['Item 1', 'Item 2', 'Item 3'], 1, '- '),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('- [ ] 1: Item 1');
    expect(output).toContain('- [ ] 2: Item 2');
    expect(output).toContain('- [ ] 3: Item 3');
  });
});