import { jest } from '@jest/globals';
import { TestPrinter } from '../index';
import { Task, Step } from '@shared/index';
import { InMemoryPrintHistoryRepository, InMemoryStepRepository } from '../../db/repositories/in-memory-repository';
import { Knex } from 'knex';
import { CommandBuilder } from '../printer-commands';
import { TestCommandExecutor } from '../command-executor';

describe('Printer', () => {
  let printHistoryRepo: InMemoryPrintHistoryRepository;
  let stepRepo: InMemoryStepRepository;
  let testPrinter: TestPrinter;
  let mockDb: jest.Mocked<Knex>;

  beforeEach(() => {
    mockDb = {} as jest.Mocked<Knex>;
    printHistoryRepo = new InMemoryPrintHistoryRepository(mockDb);
    stepRepo = new InMemoryStepRepository(mockDb);
    testPrinter = new TestPrinter(printHistoryRepo, stepRepo);
  });

  it('should record print history when printing a task', async () => {
    const task: Task = { id: 1, name: 'Test Task', group_id: 1, print_count: 0, created_at: new Date(), updated_at: new Date() };
    await testPrinter.printTask(task, mockDb);
    const allHistory = await printHistoryRepo.findAll();
    expect(allHistory.length).toBe(1);
    expect(allHistory[0]).toMatchObject({
      user_id: 1,
      task_id: task.id,
      printed_at: expect.any(Date),
    });
  });

  it('should record print history when printing a step', async () => {
    const step: Step = { id: 1, name: 'Test Step', instructions: 'Test Instructions', task_id: 1, order: 1, print_count: 0, created_at: new Date(), updated_at: new Date() };
    await testPrinter.printStep(step, mockDb);
    const allHistory = await printHistoryRepo.findAll();
    expect(allHistory.length).toBe(1);
    expect(allHistory[0]).toMatchObject({
      user_id: 1,
      step_id: step.id,
      printed_at: expect.any(Date),
    });
  });
});

describe('Command System', () => {
  let commandExecutor: TestCommandExecutor;

  beforeEach(() => {
    commandExecutor = new TestCommandExecutor();
  });

  it('should execute text commands', async () => {
    const commands = [
      CommandBuilder.text('Hello World'),
      CommandBuilder.newline(),
      CommandBuilder.text('Test'),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('Hello World');
    expect(output).toContain('Test');
  });

  it('should execute header commands', async () => {
    const commands = [
      CommandBuilder.header('Test Header'),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('[ ] Test Header');
  });

  it('should execute banner commands', async () => {
    const commands = [
      CommandBuilder.banner('=', 10),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('==========');
  });

  it('should execute barcode commands', async () => {
    const commands = [
      CommandBuilder.barcode('123456'),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('[BARCODE: 123456]');
  });

  it('should execute section commands', async () => {
    const commands = [
      CommandBuilder.section('Test Section', 'Test Content', '-', true),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('[ ] Test Section');
    expect(output).toContain('Test Content');
  });

  it('should execute step header commands', async () => {
    const commands = [
      CommandBuilder.stepHeader('Test Step', 1, 'Test Task', true),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('[ ] Step 1: Test Step');
  });

  it('should execute cut commands', async () => {
    const commands = [
      CommandBuilder.cut(true, 4),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('--- CUT ---');
  });

  it('should execute complex command sequences', async () => {
    const commands = [
      CommandBuilder.header('Complex Test'),
      CommandBuilder.banner('=', 20),
      CommandBuilder.text('Some content'),
      CommandBuilder.newline(),
      CommandBuilder.barcode('123'),
      CommandBuilder.cut(),
    ];

    await commandExecutor.executeCommands(commands);
    const output = commandExecutor.getOutput();

    expect(output).toContain('[ ] Complex Test');
    expect(output).toContain('====================');
    expect(output).toContain('Some content');
    expect(output).toContain('[BARCODE: 123]');
    expect(output).toContain('--- CUT ---');
  });
});