export interface BarcodeData {
  type: 'TASK' | 'STEP';
  id: number;
}

export class BarcodeUtils {
  // Entity type codes - using numeric prefixes
  private static readonly TASK_PREFIX = 1;
  private static readonly STEP_PREFIX = 2;

  // Format: [entity_type][padding][id]
  // Example: 100000123 for task 123, 200000456 for step 456
  private static readonly ID_PADDING = 4; // 4 digits for ID

  /**
   * Parse barcode data to extract entity type and ID
   * @param barcodeData The raw barcode data (e.g., "100000123" or "200000456")
   * @returns Parsed barcode data with type and ID, or null if invalid
   */
  static parseBarcode(barcodeData: string): BarcodeData | null {
    const numericData = barcodeData.replace(/\D/g, ''); // Remove non-digits

    if (numericData.length < 2) {
      return null;
    }

    const entityType = parseInt(numericData.charAt(0), 10);
    const id = parseInt(numericData.substring(1), 10);

    if (isNaN(id) || id <= 0) {
      return null;
    }

    if (entityType === this.TASK_PREFIX) {
      return { type: 'TASK', id };
    } else if (entityType === this.STEP_PREFIX) {
      return { type: 'STEP', id };
    }

    return null;
  }

  /**
   * Generate barcode data for a task
   * @param taskId The task ID
   * @returns Formatted barcode data (numeric only)
   */
  static generateTaskBarcode(taskId: number): string {
    return `${this.TASK_PREFIX}${taskId.toString().padStart(this.ID_PADDING, '0')}`;
  }

  /**
   * Generate barcode data for a step
   * @param stepId The step ID
   * @returns Formatted barcode data (numeric only)
   */
  static generateStepBarcode(stepId: number): string {
    return `${this.STEP_PREFIX}${stepId.toString().padStart(this.ID_PADDING, '0')}`;
  }

  /**
   * Validate if a barcode data string is valid
   * @param barcodeData The barcode data to validate
   * @returns True if valid, false otherwise
   */
  static isValidBarcode(barcodeData: string): boolean {
    return this.parseBarcode(barcodeData) !== null;
  }

  /**
   * Get the entity type from barcode data
   * @param barcodeData The barcode data
   * @returns The entity type or null if invalid
   */
  static getEntityType(barcodeData: string): 'TASK' | 'STEP' | null {
    const parsed = this.parseBarcode(barcodeData);
    return parsed?.type || null;
  }

  /**
   * Get the entity ID from barcode data
   * @param barcodeData The barcode data
   * @returns The entity ID or null if invalid
   */
  static getEntityId(barcodeData: string): number | null {
    const parsed = this.parseBarcode(barcodeData);
    return parsed?.id || null;
  }

  /**
   * Get the maximum supported ID for the current format
   * @returns Maximum ID value
   */
  static getMaxId(): number {
    return Math.pow(10, this.ID_PADDING) - 1;
  }

  /**
   * Check if an ID is within the supported range
   * @param id The ID to check
   * @returns True if valid, false otherwise
   */
  static isValidId(id: number): boolean {
    return id > 0 && id <= this.getMaxId();
  }
}