import { Printer } from '@node-escpos/core';
import USB from '@node-escpos/usb-adapter';
import { Task, Step, Printer as PrinterInterface } from '@shared/index';
import { StepRepository, PrintHistoryRepository } from '../db/repositories';
import { Knex } from 'knex';
import logger from '../logger';
import { formatUtils } from './format-utils';
import { Command, CommandBuilder } from './printer-commands';
import { SerialCommandExecutor } from './command-executor';
import {
  PRINTER_CONFIG,
  FONT_SIZES,
  BARCODE_CONFIG,
  PAPER_CONFIG,
  ALIGNMENT,
  FONT,
  STYLE,
} from './printer-constants';

export class SerialPrinter implements PrinterInterface {
  private device: USB | null = null;
  private printer: Printer<[]> | null = null;
  private printHistoryRepo: PrintHistoryRepository;
  private stepRepository: StepRepository;
  private commandExecutor: SerialCommandExecutor | null = null;

  constructor(printHistoryRepo: PrintHistoryRepository, stepRepo: StepRepository) {
    this.printHistoryRepo = printHistoryRepo;
    this.stepRepository = stepRepo;
    this.initializeDevice();
  }

  private initializeDevice() {
    try {
      this.device = new USB();
      const options = { encoding: PRINTER_CONFIG.ENCODING };
      this.printer = new Printer(this.device, options);
      this.commandExecutor = new SerialCommandExecutor(this.printer);
      logger.info('Printer device initialized successfully');
    } catch (error) {
      logger.error('Failed to initialize printer device:', error);
    }
  }

  private async openPrinter(): Promise<void> {
    if (!this.device) {
      throw new Error('Printer device not initialized');
    }

    try {
      await new Promise<void>((resolve, reject) => {
        this.device?.open((err) => {
          if (err) {
            logger.error('Failed to open printer:', err);
            reject(err);
            return;
          }
          resolve();
        });
      });
      logger.info('Printer opened successfully');
    } catch (error) {
      logger.error('Failed to open printer:', error);
      throw error;
    }
  }

  private async closePrinter(): Promise<void> {
    if (!this.printer) {
      throw new Error('Printer not initialized');
    }

    try {
      await this.printer.close();
      logger.info('Printer closed successfully');
    } catch (error) {
      logger.error('Failed to close printer:', error);
      throw error;
    }
  }

  async getTaskSteps(_db: Knex, task: Task): Promise<Step[]> {
    return await this.stepRepository.findByTaskId(task.id);
  }

  async printTask(task: Task, db: Knex): Promise<void> {
    if (!this.commandExecutor) {
      throw new Error('Printer not initialized');
    }

    const taskSteps = await this.getTaskSteps(db, task);
    logger.info(`Printing task ${task.id} with ${taskSteps.length} steps`);

    try {
      await this.openPrinter();

      const commands = [
        // Header with task name
        CommandBuilder.header(`Task: ${task.name}`),
        CommandBuilder.banner('=', PAPER_CONFIG.BANNER_LENGTH),
        CommandBuilder.align(ALIGNMENT.LEFT),

        // Task ID as barcode
        ...CommandBuilder.taskBarcodeWithAlignment(task.id),
        CommandBuilder.newline(),
        CommandBuilder.newline(),
      ];

      // Add steps
      for (let i = 0; i < taskSteps.length; i++) {
        const step = taskSteps[i];
        logger.info(`Printing step ${step.id}: name="${step.name}", instructions="${step.instructions}"`);

        commands.push(
          CommandBuilder.fontSize(FONT_SIZES.NORMAL),
          formatUtils.stepHeader(step.name, i + 1, task.name, true),
          CommandBuilder.banner('-', PAPER_CONFIG.BANNER_LENGTH),
          CommandBuilder.text(
            step.instructions || 'No instructions provided',
          ),
          CommandBuilder.newline(),
          // ...CommandBuilder.stepBarcodeWithAlignment(step.id),
          CommandBuilder.newline()
        );
      }

      commands.push(CommandBuilder.cut(true, PAPER_CONFIG.CUT_LINES));

      await this.commandExecutor.executeCommands(commands);
      await this.closePrinter();

      logger.info(`Printed task ${task.id}`);

      await this.printHistoryRepo.create({
        user_id: PRINTER_CONFIG.DEFAULT_USER_ID,
        task_id: task.id,
        printed_at: new Date(),
      });
    } catch (error) {
      logger.error('Failed to print task:', error);
      throw error;
    }
  }

  async printStep(step: Step, db: Knex): Promise<void> {
    if (!this.commandExecutor) {
      throw new Error('Printer not initialized');
    }

    logger.info(`Printing step ${step.id}: name="${step.name}", instructions="${step.instructions}"`);

    try {
      await this.openPrinter();

      // Get the task name for context
      const task = await this.stepRepository.findTaskById(step.id);
      const stepNumber = await this.stepRepository.findStepNumber(step.id);

      const headerText = formatUtils.getCheckboxText(formatUtils.stepHeader(step.name, stepNumber, task?.name));
      const stepSection = formatUtils.section(
        headerText,
        step.instructions || 'No instructions provided',
        '-'
      );

      const commands = [
        CommandBuilder.font(FONT.DEFAULT),
        CommandBuilder.align(ALIGNMENT.CENTER),
        CommandBuilder.style(STYLE.BOLD),
        CommandBuilder.fontSize(FONT_SIZES.NORMAL),
        formatUtils.stepHeader(step.name, stepNumber, task?.name),
        // CommandBuilder.text(headerText),
        CommandBuilder.newline(),
        CommandBuilder.align(ALIGNMENT.LEFT),
        CommandBuilder.fontSize(FONT_SIZES.NORMAL),
        CommandBuilder.text(step.instructions || 'No instructions provided'),
        CommandBuilder.newline(),
        CommandBuilder.newline(),
        ...CommandBuilder.stepBarcodeWithAlignment(step.id),
        CommandBuilder.cut(true, PAPER_CONFIG.CUT_LINES),
      ];

      await this.commandExecutor.executeCommands(commands);
      await this.closePrinter();

      logger.info(`Printed step ${step.id}`);

      await this.printHistoryRepo.create({
        user_id: PRINTER_CONFIG.DEFAULT_USER_ID,
        step_id: step.id,
        printed_at: new Date(),
      });
    } catch (error) {
      logger.error('Failed to print step:', error);
      throw error;
    }
  }
}
