import fs from 'fs/promises';
import path from 'path';
import { Task, Step, Printer } from '@shared/index';
import { StepRepository, PrintHistoryRepository } from '../db/repositories';
import { Knex } from 'knex';
import logger from '../logger';
import { formatUtils } from './format-utils';
import { CommandBuilder } from './printer-commands';
import { TestCommandExecutor } from './command-executor';
import {
  PAPER_CONFIG,
  FONT_SIZES,
  ALIGNMENT,
  FONT,
  STYLE,
} from './printer-constants';

export class TestPrinter implements Printer {
  private readonly outputDir: string;
  private printHistoryRepo: PrintHistoryRepository;
  private stepRepository: StepRepository;
  private commandExecutor: TestCommandExecutor;

  constructor(printHistoryRepo: PrintHistoryRepository, stepRepo: StepRepository) {
    this.outputDir = path.join(process.cwd(), 'test-output');
    this.printHistoryRepo = printHistoryRepo;
    this.stepRepository = stepRepo;
    this.commandExecutor = new TestCommandExecutor();
    this.ensureOutputDir();
  }

  private async ensureOutputDir() {
    try {
      await fs.mkdir(this.outputDir, { recursive: true });
    } catch (error) {
      logger.error('Failed to create output directory:', error);
    }
  }

  async getTaskSteps(db: Knex, task: Task): Promise<Step[]> {
    return await this.stepRepository.findByTaskId(task.id);
  }

  async printTask(task: Task, db: Knex): Promise<void> {
    const taskSteps = await this.getTaskSteps(db, task);
    const timestamp = new Date().toISOString().replace(/[:.]/g, '-');
    const filename = path.join(this.outputDir, `task-${task.id}-${timestamp}.txt`);

    const commands = [
      // Header with task name
      CommandBuilder.header(`Task: ${task.name}`),
      CommandBuilder.banner('=', PAPER_CONFIG.BANNER_LENGTH),
      CommandBuilder.align(ALIGNMENT.LEFT),

      // Task ID as barcode
      ...CommandBuilder.taskBarcodeWithAlignment(task.id),
      CommandBuilder.newline(),
    ];

    // Add steps
    for (let i = 0; i < taskSteps.length; i++) {
      const step = taskSteps[i];
      logger.info(`Printing step ${step.id}: name="${step.name}", instructions="${step.instructions}"`);

      const headerText = formatUtils.getCheckboxText(formatUtils.stepHeader(step.name, i + 1, task.name, true));
      commands.push(
        CommandBuilder.fontSize(FONT_SIZES.NORMAL),
        CommandBuilder.text(headerText),
        CommandBuilder.newline(),
        ...formatUtils.section(
          '',
          step.instructions || 'No instructions provided',
          '-'
        ),
        ...CommandBuilder.stepBarcodeWithAlignment(step.id),
        CommandBuilder.newline()
      );
    }

    commands.push(CommandBuilder.cut(true, PAPER_CONFIG.CUT_LINES));

    await this.commandExecutor.executeCommands(commands);
    const content = this.commandExecutor.getOutput();

    await fs.writeFile(filename, content);
    logger.info(`Printed task ${task.id} to ${filename}`);

    await this.printHistoryRepo.create({
      user_id: 1, // Replace with actual user ID if available
      task_id: task.id,
      printed_at: new Date(),
    });
  }

  async printStep(step: Step, db: Knex): Promise<void> {
    void db; // avoid unused parameter warning
    const timestamp = new Date().toISOString().replace(/[:.]/g, '-');
    const filename = path.join(this.outputDir, `step-${step.id}-${timestamp}.txt`);

    // Get the task name for context
    const task = await this.stepRepository.findTaskById(step.id);
    const stepNumber = await this.stepRepository.findStepNumber(step.id);

    const headerText = formatUtils.getCheckboxText(formatUtils.stepHeader(step.name, stepNumber, task?.name));

    const commands = [
      CommandBuilder.font(FONT.DEFAULT),
      CommandBuilder.align(ALIGNMENT.CENTER),
      CommandBuilder.style(STYLE.BOLD),
      CommandBuilder.fontSize(FONT_SIZES.NORMAL),
      CommandBuilder.text(headerText),
      CommandBuilder.newline(),
      CommandBuilder.align(ALIGNMENT.LEFT),
      CommandBuilder.fontSize(FONT_SIZES.NORMAL),
      CommandBuilder.text(step.instructions || 'No instructions provided'),
      CommandBuilder.newline(),
      CommandBuilder.newline(),
      ...CommandBuilder.stepBarcodeWithAlignment(step.id),
      CommandBuilder.cut(true, PAPER_CONFIG.CUT_LINES),
    ];

    await this.commandExecutor.executeCommands(commands);
    const content = this.commandExecutor.getOutput();

    await fs.writeFile(filename, content);
    logger.info(`Printed step ${step.id} to ${filename}`);

    await this.printHistoryRepo.create({
      user_id: 1, // Replace with actual user ID if available
      step_id: step.id,
      printed_at: new Date(),
    });
  }
}
